#include "aladin.h"

#define SERIAL "Input device undefined. Use \"-d\" or set $ALADIN_TTY\n"
#define USAGE1 "usage: atu [-v] [-a] [-f] [-d tty_device] -l logbook\n"
#define USAGE2 "usage: atu [-v] -r -l logbook\n"
#define USAGE  USAGE1 USAGE2

ALADIN aladins[] = {
	{ 0x1c,	"Aladin Air",		22, 1 },
	{ 0x1e,	"Aladin Sport",		22, 0 },
	{ 0x3e,	"Aladin Sport",		22, 0 },
	{ 0x1f,	"Aladin Pro",		22, 0 },
	{ 0x3f,	"Aladin Pro",		22, 0 },
	{ 0x34,	"Aladin Air X",		22, 1 },
	{ 0x44,	"Aladin Air X",		22, 1 },
	{ 0xa4,	"Aladin Air X O2",	25, 1 },
	{ 0xf4,	"Aladin Air X Nitrox",	24, 1 },
	{ 0xff,	"Aladin Pro Nitrox",	24, 0 },
	{ 0,	NULL,			22, 0 }
};

int debug = 0;

int main(int argc, char **argv) {
	char *serial  = NULL;
	char *logbook = NULL;
	int c, errflg = 0, add_all = 0, force = 0, view = 0;
	LOG *old, *new;

	/* Hey, you never know... */
	assert(sizeof(unsigned char) == 1);
	assert(sizeof(short) == 2);
	assert(sizeof(long) == 4);

	if (getenv("ALADIN_TTY") != NULL) {
		serial = strdup(getenv("ALADIN_TTY"));
	}

	while ((c = getopt(argc, argv, "varfd:l:")) != -1) {
		switch(c) {
		case 'v':
			debug++;
			break;
		case 'a':
			add_all++;
			break;
		case 'f':
			force++;
			break;
		case 'r':
			view++;
			break;
		case 'd':
			serial = strdup(optarg);
			break;
		case 'l':
			logbook = strdup(optarg);
			break;
		case '?':
			errflg++;
			break;
		}
	}

	if (errflg || (serial && view) || (!serial && !view) || !logbook) {
		if ((serial && view)) {
			fprintf(stderr, SERIAL);
		}
		fprintf(stderr, "Aladin Transfer Utility %s\n", VERSION);
		fprintf(stderr, USAGE);
		exit(1);
	}
	if (debug) {
		fprintf(stderr, "Aladin Transfer Utility %s\n", VERSION);
	}

	old = read_dtrak(logbook);
	if (view) {
		log_sort(old);
		new = renumber(old);
		dump_logbook(new);
	} else {
		new = read_aladin(serial);
		new = merge(old, new, add_all, force);
		log_sort(new);
		new = renumber(new);
	}
	write_dtrak(logbook, new);

	return(0);
}

