/*
 * Aladin dive computer transfer utility.
 * http://aeschi.ch.eu.org/aladin/
 *
 * (c) Al Smith 2001-2005.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <assert.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <termios.h>
#include <unistd.h>

#define VERSION "1.2b"

typedef unsigned char * PROFILE;

typedef union raw_log_t {
	unsigned char raw[12];
	struct log_data_t {

#define	HIGH_H	0x80
#define	HIGH_L	0x40
#define	HIGH	(HIGH_H | HIGH_L)
#define	SOS	0x20
#define	WORK	0x10
#define	DECO	0x08
#define	BTIME_H	0x04
#define	REPEAT	0x02
#define	ASCENT	0x01
		unsigned char alarm;
		unsigned char btime_l;
		unsigned char depth[2];
		unsigned char surf[2];
		unsigned char air;
		unsigned char entry[4];
		unsigned char temp;
	} data;
} RAW_LOG;

typedef union header_t {
	unsigned char raw[12];
	struct header_data_t {
		unsigned char magic[2]; /* 0xA100 */
		unsigned char serial[4];
		unsigned char count[2];
		unsigned char unk[4]; /* Initialise to 0x64001400 */
	} data;
} HEADER;

typedef struct entry_t {
	unsigned char header[2];
	unsigned char date[3];
	unsigned char tz;
	unsigned char time[2];
	unsigned char lenloc;
	unsigned char *loc;
	unsigned char lenplace;
	unsigned char *place;
	unsigned char height;
	unsigned char interval[2];
	unsigned char weather;
	unsigned char airtemp[2];
	unsigned char suit;
	unsigned char tank[2];
	unsigned char maxdepth[2];
	unsigned char divetime[2];
	unsigned char watertemp[2];
	unsigned char air[2];
	unsigned char divetype[2];
	unsigned char actions[2];
	unsigned char lenact;
	unsigned char *act;
	unsigned char lenbuddy;
	unsigned char *buddy;
	unsigned char lencomment;
	unsigned char *comment;
	unsigned char alarm[2];
	unsigned char divenum[2];
	unsigned char timestamp[4];
	unsigned char type;
	unsigned char info;
	unsigned char unk[6]; /* Initialise to zero */
	unsigned char lenpro[2];
	PROFILE pro;
} ENTRY;

typedef struct log_t {
	HEADER header;
	ENTRY *entries;
	unsigned int count;
} LOG;

typedef union info_t {
	unsigned char raw[52];
	struct info_data_t {
		unsigned char aladin;
		unsigned char unk1[21];

#define	BEEP	0x02
#define	UNITS	0x01
		unsigned char settings;
		unsigned char o2pp;
		unsigned char unk2[10];
		unsigned char reserve;
		unsigned char unk3[12];
		unsigned char sens;
		unsigned char unk4; /* always 0x0b? */
		unsigned char serial[3];
	} data;
} INFO;

typedef union status_t {
	unsigned char raw[14];
	struct status_data_t {
		unsigned char batt;
		unsigned char dives[3];
		unsigned char off_log;
		unsigned char profiles;
		unsigned char eop[2];
		unsigned char time[4];
		unsigned char csum[2];
	} data;
} STATUS;

typedef struct aladin_t {
	unsigned char	type;
	char		*name;
	int		deco_len;
	int		air;
} ALADIN;

extern int debug;
extern ALADIN aladins[];

extern LOG * read_dtrak(char *);
extern LOG * read_aladin(char *);
extern LOG * reformat(const unsigned char *, unsigned long);
extern LOG * merge(LOG *, LOG *, int, int);
extern LOG * renumber(LOG *);
extern void log_sort(LOG *);
extern void write_dtrak(char *, LOG *);
extern void dump_raw_data(const unsigned char *, unsigned long);
extern void dump_logbook(LOG *);

