#include "aladin.h"

static void dump_raw_log(const RAW_LOG *log, unsigned long timecorr) {
	unsigned long i;
	double j;

	printf("\tHeight:\t\t\t%s\n",
		(log->data.alarm & HIGH_H) ?
			((log->data.alarm & HIGH_L) ? "2700m-4000m" : "1700m-2700m")
		:
			((log->data.alarm & HIGH_L) ? "900m-1700m" : "0m-900"));
	printf("\tAlarms:\t\t%s%s%s%s\n",
		(log->data.alarm & SOS) ? "\tSOS" : "",
		(log->data.alarm & WORK) ? "\tWork" : "",
		(log->data.alarm & DECO) ? "\tDecompression" : "",
		(log->data.alarm & ASCENT) ? "\tAscent" : "");

	printf("\tDive time:\t\t%s%x\n",
		 (log->data.alarm & BTIME_H) ? "1" : "", log->data.btime_l);

	j = (log->data.depth[0] * 256 + log->data.depth[1]) / 409.6;
	printf("\tMax depth:\t\t%.1f m\n", j);

	if (log->data.alarm & REPEAT) {
		printf("\tSurface interval:\t%x:%02x\n",
			 log->data.surf[0], log->data.surf[1]);
	}

	printf("\tAir consumption:\t%.0f bar\n", log->data.air * 1.37634409);
	
	i  = log->data.entry[0]; i <<= 8;
	i += log->data.entry[1]; i <<= 8;
	i += log->data.entry[2]; i <<= 8;
	i += log->data.entry[3];
	i >>= 1;
	printf("\tRaw entry time:\t\t%ld\n", i);

	i += 757382400;
	i += timecorr;
	printf("\tEntry time:\t\t%s", ctime((const time_t *) &i));

	printf("\tWater temp:\t\t%d c\n", log->data.temp / 4);

	return;
}

static void dump_profile(const unsigned char *buf, unsigned int first, int skip, ALADIN *aladin) {
	int i, j, byte, o2;
	int secs;
	double depth;

	printf("\tProfile %d\n", skip + 1);

	for(i = 0; i < 0x600; i++) {
		byte = (i + first + 1) % 0x600;
		if (buf[byte] == 0xff) {
			if (skip) {
				skip--;
			} else {
				if (debug) {
					printf("Found profile at %#x (%#x)\n", byte, i);
				}
				break;
			}
		}
	}

	if (aladin->type == 0xf4 || aladin->type == 0xff) {
		o2 = buf[byte+24] & 0x0f;
		if (o2 == 0) {
			o2 = 21;
		} else {
			o2 = (o2 - 1) * 2 + 22;
		}
		printf("O2: %d\n", o2);
	}

	for(secs = 20, j = i + aladin->deco_len + 1; j < 0x600; secs += 20, j++, j++) {
		byte = (j + first + 1) % 0x600;
		if (buf[byte] == 0xff) {
			if (debug) {
				printf("Profile ends at %#x (%#x)\n", byte, j);
			}
			break;
		}
		depth = (buf[byte] * 256 + buf[byte+1]) / 409.6;
		printf("\t%d:%02d\t%.1f m\t%c%c%c%c%c%c\n",
			secs/60, secs%60, depth,
			buf[byte+1] & 0x20 ? 't' : '.',
			buf[byte+1] & 0x10 ? 'w' : '.',
			buf[byte+1] & 0x08 ? 'd' : '.',
			buf[byte+1] & 0x04 ? 'a' : '.',
			buf[byte+1] & 0x02 ? 'r' : '.',
			buf[byte+1] & 0x01 ? 's' : '.');
		if (secs % 60 == 0) {
			printf("\tDeco:\tWork:%d\t%s\tBubble:%d\n",
				(buf[byte+2] & 0x70) >> 4,
				buf[byte+2] & 0x80 ? "Warm" :
					(buf[byte+2] & 0x08 ? "Cold" : ""),
				(buf[byte+2] & 0x07));
			j++;
		}
	}

	return;
}

void dump_raw_data(const unsigned char *buf, unsigned long now) {
	int c;
	unsigned long i;
	unsigned long timecorr;
	STATUS status;
	INFO info;
	RAW_LOG log;
	ALADIN *aladin;

	memcpy(status.raw, buf + 0x7f0, sizeof(status.raw));

	printf("Battery:\t%d%%\n", status.data.batt * 99 / 255);

	i  = status.data.dives[0]; i <<= 8;
	i += status.data.dives[1]; i <<= 8;
	i += status.data.dives[2];
	printf("Total dives:\t%ld\n", i);

	printf("Newest log at:\t%#x\n", (status.data.off_log - 1) * 12 + 0x600);
	printf("Profile count:\t%d\n", status.data.profiles);

	i = status.data.eop[1]; i >>= 1;
	i *= 256; i += status.data.eop[0]; i &= 0x7ff;
	printf("Profile end:\t%#lx\n", i);

	i  = status.data.time[0]; i <<= 8;
	i += status.data.time[1]; i <<= 8;
	i += status.data.time[2]; i <<= 8;
	i += status.data.time[3];
	i >>= 1;
	i += 757382400;
	printf("Transfer time:\t%ld\n", i);
	printf("Transfer time:\t%s", ctime((const time_t *) &i));
	printf("Actual time:\t%ld\n", now);
	printf("Actual time:\t%s", ctime((const time_t *) &now));
	timecorr = now - i;
	printf("Time correction:\t%ld\n", timecorr);

	i  = status.data.csum[1]; i <<= 8;
	i += status.data.csum[0];
	printf("Checksum1:\t%#lx\n", i);

	for(c = 0, i = 0; c < 0x7fc; c++) {
		i += buf[c];
	}
	i += 0x1fe;
	i %= (1 << 16);
	printf("Checksum2:\t%#lx\n", i);
	printf("\n"); 
		

	memcpy(info.raw, buf + 0x7bc, sizeof(info.raw));
		
	for(aladin = aladins; aladin->name; aladin++) {
		if (info.data.aladin == aladin->type) {
			break;
		}
	}
	printf("Aladin type:\t%s\n", aladin->name ? aladin->name : "");
	
	printf("Beeper:\t\t%s\n", info.data.settings & 0x02 ? "On" : "Off");
	printf("Units:\t\t%s\n", info.data.settings & 0x01 ? "Imperial" : "Metric");     

	printf("ppO2:\t\t%d\n", info.data.o2pp);
	printf("Reserve:\t%d bar\n", info.data.reserve);
	printf("Sensitivity:\t%d%%\n", info.data.sens);

	i  = info.data.serial[0]; i <<= 8; 
	i += info.data.serial[1]; i <<= 8; 
	i += info.data.serial[2];
	printf("Serial number:\t%#lx\n", i);

	for(c = 0; c < 37; c++) {
		i = (status.data.off_log + c) * 12 % 444 + 0x600;
		printf("%d: %#lx\n", c, i);
		memcpy(log.raw, buf + i, sizeof(log.raw));
		dump_raw_log(&log, timecorr);

		if (c >= 37 - status.data.profiles) {
			i = status.data.eop[1]; i >>= 1;
			i *= 256; i += status.data.eop[0]; i &= 0x7ff;
			dump_profile(buf, i, c - (37 - status.data.profiles), aladin);
		}
	}

	return;
}       

void dump_logbook(LOG *log) {
	int i, j, lenpro;

	printf("Dive count: %d\n\n", log->header.data.count[1] * 256 + log->header.data.count[0]);

	for(i = 0; i < log->count; i++) {
		printf("Dive %d\n", i);
		printf("Date: %d\n", log->entries[i].date[2] * 256*256 + log->entries[i].date[1] * 256 + log->entries[i].date[0]);
		printf("TZ: %d\n", log->entries[i].tz);
		printf("Time: %d\n", log->entries[i].time[1] * 256 + log->entries[i].time[0]);
		printf("Loc: %s\n", log->entries[i].loc);
		printf("Place: %s\n", log->entries[i].place);
		printf("Height: %d\n", log->entries[i].height);
		printf("Interval: %d\n", log->entries[i].interval[1] * 256 + log->entries[i].interval[0]);
		printf("Weather: %d\n", log->entries[i].weather);
		printf("Airtemp: %d\n", log->entries[i].airtemp[1] * 256 + log->entries[i].airtemp[0]);
		printf("Suit: %d\n", log->entries[i].suit);
		printf("Tank: %d\n", log->entries[i].tank[1] * 256 + log->entries[i].tank[0]);
		printf("Maxdepth: %d\n", log->entries[i].maxdepth[1] * 256 + log->entries[i].maxdepth[0]);
		printf("Divetime: %d\n", log->entries[i].divetime[1] * 256 + log->entries[i].divetime[0]);
		printf("Water temp: %d\n", log->entries[i].watertemp[1] * 256 + log->entries[i].watertemp[0]);
		printf("Air consumption: %d\n", log->entries[i].air[1] * 256 + log->entries[i].air[0]);
		printf("Dive type: %d\n", log->entries[i].divetype[1] * 256 + log->entries[i].divetype[0]);
		printf("Actions: %d\n", log->entries[i].actions[1] * 256 + log->entries[i].actions[0]);
		printf("Descriptions: %s\n", log->entries[i].act);
		printf("Buddies: %s\n", log->entries[i].buddy);
		printf("Comments: %s\n", log->entries[i].comment);
		printf("Alarms: %d\n", log->entries[i].alarm[1] * 256 + log->entries[i].alarm[0]);
		printf("Dive number: %d\n", log->entries[i].divenum[1] * 256 + log->entries[i].divenum[0]);
		printf("Timestamp: %d\n", log->entries[i].timestamp[3] * 256*256*256 + log->entries[i].timestamp[2] * 256*256 + log->entries[i].timestamp[1] * 256 + log->entries[i].timestamp[0]);
		printf("Type: %d\n", log->entries[i].type);
		printf("Info: %d\n", log->entries[i].info);

		lenpro  = log->entries[i].lenpro[1] * 256;
		lenpro += log->entries[i].lenpro[0];
		if (lenpro > 0) {
			for(j = 0; j < lenpro; j++) {
				printf("Profile[%d]: %d\n", j, log->entries[i].pro[j]);
			}
		}
		printf("\n");
	}

	return;
}

