#include "aladin.h"

static void do_read(int fd, unsigned char *buf, int len) {
	int rc;

	rc = read(fd, buf, len);

	if (rc < 0) {
		perror("write");
		exit(1);
	} else if (rc != len) {
		fprintf(stderr, "Short read.\n");
		exit(1);
	}

	return;
}

static void do_malloc(unsigned char ** buf, int len) {
	*buf = (unsigned char *) malloc(len * sizeof(unsigned char));
	if (!*buf) {
		perror("malloc");
		exit(1);
	}
}

static void do_write(int fd, unsigned char *buf, int len) {
	if (write(fd, buf, len) < 0) {
		perror("write");
		exit(1);
	}
	return;
}

LOG * read_dtrak(char *filename) {
	int i, fd, lenpro;
	LOG *log;

	log = (LOG *) malloc(sizeof(LOG));
	if (!log) {
		perror("malloc");
		exit(1);
	}

	if ((fd = open(filename, O_RDONLY)) < 0) {
		if (errno != ENOENT) {
			fprintf(stderr, "%s: ", filename);
			perror("open");
			exit(1);
		} else {
			return(NULL);
		}
	}

	do_read(fd, log->header.raw, sizeof(log->header.raw));

	log->count  = log->header.data.count[1] * 256;
	log->count += log->header.data.count[0];

	log->entries = (ENTRY *) malloc(log->count * sizeof(ENTRY));
	if (!log->entries) {
		perror("malloc");
		exit(1);
	}

	for(i = 0; i < log->count; i++) {
		do_read(fd, log->entries[i].header, 2);
		do_read(fd, log->entries[i].date, 3);
		do_read(fd, &log->entries[i].tz, 1);
		do_read(fd, log->entries[i].time, 2);
		do_read(fd, &log->entries[i].lenloc, 1);
		if (log->entries[i].lenloc > 0) {
			do_malloc(&log->entries[i].loc, log->entries[i].lenloc);
			do_read(fd, log->entries[i].loc, log->entries[i].lenloc);
		}
		do_read(fd, &log->entries[i].lenplace, 1);
		if (log->entries[i].lenplace > 0) {
			do_malloc(&log->entries[i].place, log->entries[i].lenplace);
			do_read(fd, log->entries[i].place, log->entries[i].lenplace);
		}
		do_read(fd, &log->entries[i].height, 1);
		do_read(fd, log->entries[i].interval, 2);
		do_read(fd, &log->entries[i].weather, 1);
		do_read(fd, log->entries[i].airtemp, 2);
		do_read(fd, &log->entries[i].suit, 1);
		do_read(fd, log->entries[i].tank, 2);
		do_read(fd, log->entries[i].maxdepth, 2);
		do_read(fd, log->entries[i].divetime, 2);
		do_read(fd, log->entries[i].watertemp, 2);
		do_read(fd, log->entries[i].air, 2);
		do_read(fd, log->entries[i].divetype, 2);
		do_read(fd, log->entries[i].actions, 2);
		do_read(fd, &log->entries[i].lenact, 1);
		if (log->entries[i].lenact > 0) {
			do_malloc(&log->entries[i].act, log->entries[i].lenact);
			do_read(fd, log->entries[i].act, log->entries[i].lenact);
		}
		do_read(fd, &log->entries[i].lenbuddy, 1);
		if (log->entries[i].lenbuddy > 0) {
			do_malloc(&log->entries[i].buddy, log->entries[i].lenbuddy);
			do_read(fd, log->entries[i].buddy, log->entries[i].lenbuddy);
		}
		do_read(fd, &log->entries[i].lencomment, 1);
		if (log->entries[i].lencomment > 0) {
			do_malloc(&log->entries[i].comment, log->entries[i].lencomment);
			do_read(fd, log->entries[i].comment, log->entries[i].lencomment);
		}
		do_read(fd, log->entries[i].alarm, 2);
		do_read(fd, log->entries[i].divenum, 2);
		do_read(fd, log->entries[i].timestamp, 4);
		do_read(fd, &log->entries[i].type, 1);
		do_read(fd, &log->entries[i].info, 1);
		do_read(fd, log->entries[i].unk, 6);
		do_read(fd, log->entries[i].lenpro, 2);
		lenpro  = log->entries[i].lenpro[1] * 256;
		lenpro += log->entries[i].lenpro[0];
		if (lenpro > 0) {
			do_malloc(&log->entries[i].pro, lenpro);
			do_read(fd, log->entries[i].pro, lenpro);
		}
	}

	close(fd);
	return(log);
}

void write_dtrak(char *filename, LOG *log) {
	int i, fd, lenpro;

	if ((fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0644)) < 0) {
		fprintf(stderr, "%s: ", filename);
		perror("open");
		exit(1);
	}

	/* header */
	do_write(fd, log->header.raw, sizeof(log->header.raw));

	for(i = 0; i < log->count; i++) {
		do_write(fd, log->entries[i].header, 2);
		do_write(fd, log->entries[i].date, 3);
		do_write(fd, &log->entries[i].tz, 1);
		do_write(fd, log->entries[i].time, 2);
		do_write(fd, &log->entries[i].lenloc, 1);
		if (log->entries[i].lenloc > 0) {
			do_write(fd, log->entries[i].loc, log->entries[i].lenloc);
		}
		do_write(fd, &log->entries[i].lenplace, 1);
		if (log->entries[i].lenplace > 0) {
			do_write(fd, log->entries[i].place, log->entries[i].lenplace);
		}
		do_write(fd, &log->entries[i].height, 1);
		do_write(fd, log->entries[i].interval, 2);
		do_write(fd, &log->entries[i].weather, 1);
		do_write(fd, log->entries[i].airtemp, 2);
		do_write(fd, &log->entries[i].suit, 1);
		do_write(fd, log->entries[i].tank, 2);
		do_write(fd, log->entries[i].maxdepth, 2);
		do_write(fd, log->entries[i].divetime, 2);
		do_write(fd, log->entries[i].watertemp, 2);
		do_write(fd, log->entries[i].air, 2);
		do_write(fd, log->entries[i].divetype, 2);
		do_write(fd, log->entries[i].actions, 2);
		do_write(fd, &log->entries[i].lenact, 1);
		if (log->entries[i].lenact > 0) {
			do_write(fd, log->entries[i].act, log->entries[i].lenact);
		}
		do_write(fd, &log->entries[i].lenbuddy, 1);
		if (log->entries[i].lenbuddy > 0) {
			do_write(fd, log->entries[i].buddy, log->entries[i].lenbuddy);
		}
		do_write(fd, &log->entries[i].lencomment, 1);
		if (log->entries[i].lencomment > 0) {
			do_write(fd, log->entries[i].comment, log->entries[i].lencomment);
		}
		do_write(fd, log->entries[i].alarm, 2);
		do_write(fd, log->entries[i].divenum, 2);
		do_write(fd, log->entries[i].timestamp, 4);
		do_write(fd, &log->entries[i].type, 1);
		do_write(fd, &log->entries[i].info, 1);
		do_write(fd, log->entries[i].unk, 6);
		do_write(fd, log->entries[i].lenpro, 2);
		lenpro  = log->entries[i].lenpro[1] * 256;
		lenpro += log->entries[i].lenpro[0];
		if (lenpro > 0) {
			do_write(fd, log->entries[i].pro, lenpro);
		}
	}
	
	do_write(fd, "\0\0", 2);

	close(fd);

	return;
}

