#include "aladin.h"

static int isleap(int year) {
	if (year % 4) return(0);
	if (!(year % 100) && year % 400) return(0);
	if (!(year % 4000)) return(0);
	return(1);
}

LOG * reformat(const unsigned char *buf, unsigned long now) {
	unsigned long i, j, k;
	unsigned long start, stop, byte, skip, next, lstart, len;
	unsigned long days, mins, depth, timecorr, temp, air;
	struct tm *tp;

	RAW_LOG raw_log;
	STATUS status;
	INFO info;
	LOG *log;
	ALADIN *aladin;

	memcpy(status.raw, buf + 0x7f0, sizeof(status.raw));
	memcpy(info.raw, buf + 0x7bc, sizeof(info.raw));

	i  = status.data.time[0]; i <<= 8;
	i += status.data.time[1]; i <<= 8;
	i += status.data.time[2]; i <<= 8;
	i += status.data.time[3];
	i >>= 1;
	i += 757382400;
	timecorr = now - i;
	if (debug >= 2) {
		printf("Transfer time:\t%s", ctime((const time_t *) &i));
		printf("Actual time:\t%s", ctime((const time_t *) &now));
		printf("Time correction:\t%ld\n", timecorr);
	}

	log = (LOG *) malloc(sizeof(LOG));
	if (!log) {
		perror("malloc");
		exit(1);
	}

	log->header.data.magic[0] = 0xa1;
	log->header.data.magic[1] = 0x00;

	i  = info.data.serial[0]; i <<= 8;
	i += info.data.serial[1]; i <<= 8;
	i += info.data.serial[2];
	i -= 0x10000; /* Don't know why this has to happen */

	log->header.data.serial[0] = (unsigned char) (i >> 0)  & 0xff;
	log->header.data.serial[1] = (unsigned char) (i >> 8)  & 0xff;
	log->header.data.serial[2] = (unsigned char) (i >> 16) & 0xff;
	log->header.data.serial[3] = (unsigned char) (i >> 24) & 0xff;

	/* Count log entries */
	log->count = 0;
	for(i = 0; i < 37; i++) {
		/* Compute offset */
		j = (status.data.off_log + i) * 12 % 444 + 0x600;
		memcpy(raw_log.raw, buf + j, sizeof(raw_log.raw));

		/* Assume that a dive with a max depth of 0m is empty */
		j = (raw_log.data.depth[0] * 256 + raw_log.data.depth[1]);
		if (j != 0) log->count++;
	}

	log->header.data.count[0] = (unsigned char) (log->count >> 0)  & 0xff;
	log->header.data.count[1] = (unsigned char) (log->count >> 8)  & 0xff;

	log->header.data.unk[0] = 0x64;
	log->header.data.unk[1] = 0x00;
	log->header.data.unk[2] = 0x14;
	log->header.data.unk[3] = 0x00;

	for(aladin = aladins; aladin->name; aladin++) {
		if (info.data.aladin == aladin->type) {
			break;
		}
	}

	log->entries = (ENTRY *) malloc(log->count * sizeof(ENTRY));
	if (!log->entries) {
		perror("malloc");
		exit(1);
	}

	next = 0;
	for(i = 0; i < 37; i++) {
		/* Compute offset */
		j = (status.data.off_log + i) * 12 % 444 + 0x600;
		memcpy(raw_log.raw, buf + j, sizeof(raw_log.raw));

		/* Assume that a dive with a max depth of 0m is empty */
		j = (raw_log.data.depth[0] * 256 + raw_log.data.depth[1]);
		if (j == 0) continue;

		log->entries[next].header[0] = 0xa0;
		log->entries[next].header[1] = 0x00;

		j  = raw_log.data.entry[0]; j <<= 8;
		j += raw_log.data.entry[1]; j <<= 8;
		j += raw_log.data.entry[2]; j <<= 8;
		j += raw_log.data.entry[3];
		j >>= 1;
		j += timecorr;
		j += 757382400; /* midnight, Jan 1st 1994 */

		/* j now in unix time format (secs since 1970) */
		tp = localtime((time_t *) &j);

		days = 0;
		for(k = 1600; k < tp->tm_year+1900; k++) {
			days += 365 + isleap((int) k);
		}
		days += tp->tm_yday;

		/* days since 1.1.1600 */
		log->entries[next].date[0] = (unsigned char) days & 0xff;
		log->entries[next].date[1] = (unsigned char) (days >> 8) & 0xff;
		log->entries[next].date[2] = (unsigned char) (days >> 16) & 0xff;

		log->entries[next].tz = 0x00;

		mins = tp->tm_hour * 60 + tp->tm_min;

		log->entries[next].time[0] = (unsigned char) mins & 0xff;
		log->entries[next].time[1] = (unsigned char) (mins >> 8) & 0xff;

		log->entries[next].lenloc = 0;
		log->entries[next].loc = NULL;
		log->entries[next].lenplace = 0;
		log->entries[next].place = NULL;

		log->entries[next].height = ((raw_log.data.alarm & HIGH) >> 6) + 1;

		if (raw_log.data.alarm & REPEAT) {
			mins =  ((raw_log.data.surf[0] & 0xf0) >> 4) * 10;
			mins +=  (raw_log.data.surf[0] & 0x0f);
			mins *= 60;
			mins += ((raw_log.data.surf[1] & 0xf0) >> 4) * 10;
			mins +=  (raw_log.data.surf[1] & 0x0f);

			log->entries[next].interval[0] = (unsigned char) mins & 0xff;
			log->entries[next].interval[1] = (unsigned char) (mins >> 8) & 0xff;
		} else {
			log->entries[next].interval[0] = 0xff;
			log->entries[next].interval[1] = 0x7f;
		}

		log->entries[next].weather = 0;

		log->entries[next].airtemp[0] = 0xff;
		log->entries[next].airtemp[1] = 0x7f;

		log->entries[next].suit = 0;

		log->entries[next].tank[0] = 0xff;
		log->entries[next].tank[1] = 0x7f;

		depth = raw_log.data.depth[0] * 256 + raw_log.data.depth[1];
		depth = (unsigned long) (((double) depth) / 4.096);

		log->entries[next].maxdepth[0] = (unsigned char) depth & 0xff;
		log->entries[next].maxdepth[1] = (unsigned char) (depth >> 8) & 0xff;

		mins =   (raw_log.data.alarm & BTIME_H) ? 100 : 0;
		mins += ((raw_log.data.btime_l & 0xf0) >> 4) * 10;
		mins +=  (raw_log.data.btime_l & 0x0f);

		log->entries[next].divetime[0] = (unsigned char) mins & 0xff;
		log->entries[next].divetime[1] = (unsigned char) (mins >> 8) & 0xff;

		temp = raw_log.data.temp * 25;

		log->entries[next].watertemp[0] = (unsigned char) temp & 0xff;
		log->entries[next].watertemp[1] = (unsigned char) (temp >> 8) & 0xff;

		if (raw_log.data.air > 0) {
			air = (unsigned long) raw_log.data.air;
			if (aladin->type != 244) {
				air = air * 137.8951;
				air = air - air % 100;
			} else {
				air = air * 100;
			}

			log->entries[next].air[0] = (unsigned char) air & 0xff;
			log->entries[next].air[1] = (unsigned char) (air >> 8) & 0xff;
			log->entries[next].info = 0x00;
		} else {
			log->entries[next].air[0] = 0xff;
			log->entries[next].air[1] = 0x7f;
			log->entries[next].info = 0x01;

		}

		if (raw_log.data.alarm & REPEAT) {
			log->entries[next].divetype[0] = 0x02;
		} else {
			log->entries[next].divetype[0] = 0x01;
		}
		log->entries[next].divetype[1] = 0x00;

		log->entries[next].actions[0] = 0x00;
		log->entries[next].actions[1] = 0x00;

		log->entries[next].lenact = 0x00;
		log->entries[next].act = NULL;
		
		log->entries[next].lenbuddy = 0x00;
		log->entries[next].buddy = NULL;

		log->entries[next].lencomment = 0x00;
		log->entries[next].comment = NULL;

		log->entries[next].alarm[0] = raw_log.data.alarm;
		log->entries[next].alarm[1] = 0x00;

		log->entries[next].divenum[0] = (unsigned char) (next+1) & 0xff;
		log->entries[next].divenum[1] = (unsigned char) ((next+1) >> 8) & 0xff;

		log->entries[next].timestamp[0] = raw_log.data.entry[3];
		log->entries[next].timestamp[1] = raw_log.data.entry[2];
		log->entries[next].timestamp[2] = raw_log.data.entry[1];
		log->entries[next].timestamp[3] = raw_log.data.entry[0];

		log->entries[next].type = info.data.aladin;

		log->entries[next].unk[0] = 0x00;
		log->entries[next].unk[1] = 0x00;
		log->entries[next].unk[2] = 0x00;
		log->entries[next].unk[3] = 0x00;
		log->entries[next].unk[4] = 0x00;
		log->entries[next].unk[5] = 0x00;

		if (i >= 37 - status.data.profiles) {
			j = status.data.eop[1]; j >>= 1;
                        j *= 256; j += status.data.eop[0]; j &= 0x7ff;

			skip = i - (37 - status.data.profiles);

			for(k = 0; k < 0x600; k++) {
				byte = (k + j + 1) % 0x600;
				if (buf[byte] == 0xff) {
					if (skip) {
						skip--;
					} else {
						break;
					}
				}
			}
			start = byte + 1;
			lstart = k + 1;
			for(k = k + 1; k < 0x600; k++) {
				byte = (k + j + 1) % 0x600;
				if (buf[byte] == 0xff) {
					break;
				}
			}
			stop = byte;
			len = k - lstart;
			log->entries[next].lenpro[0] = (unsigned char) (len & 0xff);
			log->entries[next].lenpro[1] = (unsigned char) ((len & 0xff00) >> 8);
			log->entries[next].pro = (PROFILE) malloc(len * sizeof(PROFILE));
			if (!log->entries[next].pro) {
				perror("malloc");
				exit(1);
			}
			if (stop > start) {
				memcpy(log->entries[next].pro, buf + start, len);
			} else {
				memcpy(log->entries[next].pro, buf + start, 0x600 - start);
				memcpy(log->entries[next].pro + (0x600 - start), buf, len + (0x600 - start));
			}

			if (aladin->type == 0xf4 || aladin->type == 0xff) {
				if ((log->entries[next].pro[24] & 0xf) != 0) {
					log->entries[next].divetype[1] |= 0x01;
				}
			}

		} else {
			log->entries[next].lenpro[0] = 0x00;
			log->entries[next].lenpro[1] = 0x00;
			log->entries[next].pro = NULL;
		}

		next++;
	}

	return(log);
}

