#include "aladin.h"

static unsigned long data_parse(unsigned char *data, int len) {
	int i;
	unsigned long rc = 0;

	for(i = len-1; i >= 0; i--) {
		rc += *(data + i) << 8*i;
	}

	return(rc);
}

static int entry_comp(const void *a, const void *b) {
	ENTRY *i, *j;
	unsigned long p, q;

	i = (ENTRY *) a;
	j = (ENTRY *) b;

	p = data_parse(i->timestamp, 4);
	q = data_parse(j->timestamp, 4);

	if (p != 0 && q != 0) {
		if (p > q) return(1);
		if (p < q) return(-1);
		return(0);
	}

	p = data_parse(i->date, 3);
	q = data_parse(j->date, 3);

	if (p > q) return(1);
	if (p < q) return(-1);

	p = data_parse(i->time, 2);
	q = data_parse(j->time, 2);

	if (p > q) return(1);
	if (p < q) return(-1);
	return(0);
}

LOG * merge(LOG *old, LOG *new, int add_all, int force) {
	int i, j, next;
	ENTRY *entries;
	unsigned long last = 0;

	if (!old) return(new);

	if (old->header.data.serial[0] != new->header.data.serial[0] ||
	    old->header.data.serial[1] != new->header.data.serial[1] ||
	    old->header.data.serial[2] != new->header.data.serial[2] ||
	    old->header.data.serial[3] != new->header.data.serial[3]) {
		fprintf(stderr, "Warning: new dives are from a difference computer.\n");
		if (!force) {
			fprintf(stderr, "Use -f to force addition.\n");
			exit(1);
		}
		old->header.data.serial[0] = new->header.data.serial[0];
		old->header.data.serial[1] = new->header.data.serial[1];
		old->header.data.serial[2] = new->header.data.serial[2];
		old->header.data.serial[3] = new->header.data.serial[3];
	}

	entries = (ENTRY *) malloc((old->count * new->count) * sizeof(ENTRY));
	if (!entries) {
		perror("malloc");
		exit(1);
	}

	for(i = next = 0; i < old->count; i++) {
		entries[next++] = old->entries[i];
		last = data_parse(old->entries[i].timestamp, 4) > last ? data_parse(old->entries[i].timestamp, 4) : last;
	}

	for(i = 0; i < new->count; i++) {
		if (!add_all && last > data_parse(new->entries[i].timestamp, 4)) {
			continue;
		}
		for(j = 0; j < next; j++) {
			if (data_parse(new->entries[i].timestamp, 4) == data_parse(entries[j].timestamp, 4)) {
				break;
			}
		}
		if (j == next) {
			entries[next++] = new->entries[i];
		}
	}

	old->count = next;
	old->header.data.count[0] = (unsigned char) (next >> 0) & 0xff;
	old->header.data.count[1] = (unsigned char) (next >> 8) & 0xff;
	old->entries = entries;

	return(old);
}

void log_sort(LOG *log) {
	qsort(log->entries, log->count, sizeof(ENTRY), entry_comp);
	return;
}

LOG * renumber(LOG *log) {
	int i, j;

	/* update entry dive counts, starting from number in old->entries[0] */
	j  = log->entries[0].divenum[1];
	j *= 256;
	j += log->entries[0].divenum[0];

	for(i = 1; i < log->count; i++) {
		log->entries[i].divenum[0] = (unsigned char) (++j) & 0xff;
		log->entries[i].divenum[1] = (unsigned char) (j >> 8) & 0xff;
	}

	return(log);
}

